/* Written by Morgoth DBMA, morgothdbma@o2.pl
 This is part of PgXexplorer software, Open Source
 on BSD licence, Libraries(interaces) used:
 GNU GCC, AS (all stuff needed to compile C source into executable binary)
 LibPQ-FE from PostgreSQL, GTK (GIMP Toolkit)
 written in VIM editor, ctags used, CVS used
 Currently only one author: MOrgoth DBMA
 FILE: query.c */
#include <libpq-fe.h>
#include "consoleout.h"
#include "query.h"
#include "signals.h"
/*FIXME dodac obsluge debug, error ==> DlgWindow*/

PGconn* connect_db(char* arg)
{
 PGconn* conn;
 char* strarg;
 conn=0;
 strarg = (char*)malloc(strlen(arg)+16);
 debug("\nconnect to database %s\n", arg);
 sprintf(strarg,"dbname=%s", arg);
 alarm_clock(10);
 conn = PQconnectdb(strarg);
 alarm_stop();
 if (PQstatus(conn) == CONNECTION_BAD)
   {
    error("\nCannot connect to DB:%s\n", arg);
    ERR
    return NULL;
   }
 free(strarg);
 return conn;
}


PGconn* gtk_connect_db(char* arg, char* errstr)
{
 PGconn* conn;
 char* strarg;
 conn=0;
 strarg = (char*)malloc(strlen(arg)+16);
 sprintf(strarg,"dbname=%s", arg);
 alarm_clock(10);
 conn = PQconnectdb(strarg);
 alarm_stop();
 if (PQstatus(conn) == CONNECTION_BAD)
  {
   sprintf(errstr, "Connection string:\ndbname=%s failed.", arg);
   free(strarg);
   return NULL;
  }
 free(strarg);
 return conn;
}


void clear_result(PGresult** r)
{
 if (*r) PQclear(*r);
 debug("clear result\n");
 *r=0;
}


void disconnect_db(PGconn** c)
{
 debug("disconnect DB\n");
 if (*c) PQfinish(*c);
 *c = NULL;
}


PGresult* execute_silent_query(PGconn* c,  char* query)
{
 PGresult* r;
 int stat;
 debug("SQLquery: \"%s\"\n", query);
 if (!query)
   {
    debug("NULL query passed\n");
    return NULL;
   }
 if (!c) return NULL;
 r= PQexec(c, query);
 if (!r)
   {
    debug("PQresult is NULL %s\n", PQerrorMessage(c));
    return NULL;
   }
 stat = PQresultStatus(r);
 if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
   {
    debug("PQexec error: %s %s",PQcmdStatus(r),PQresultErrorMessage(r));
    debug("Failed query was: \"%s\"\n", query);
    return NULL;
   }
 return r;
}


char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, char* fmt,...)
{
 va_list ap;
 char*** ret;
 char msg[MAX_QUERY_LENGTH];
 va_start(ap, fmt);
 vsprintf(msg,fmt,ap);
 va_end(ap);
 execute_printf(c, &ret, row, col, errstr, msg);
 return ret;
}


void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, char* fmt,...)
{
 int stat;
 va_list ap;
 char query[MAX_QUERY_LENGTH];
 PGresult* r;
 va_start(ap, fmt);
 vsprintf(query,fmt,ap);
 va_end(ap);
 r=NULL;
 if (!c)
   {
    *res = NULL;
    *row = *col = -1;
    strcpy(errstr, "Not connected to DB");
    return;
   }
 if (!query)
   {
    *res = NULL;
    *row = *col = -1;
    strcpy(errstr, "NULL query passed to DB");
    return;
   }
 r= PQexec(c, query);
 if (!r)
   {
    sprintf(errstr,"PQresult is NULL %s\nQuery was:\n%s\n", PQerrorMessage(c), query);
    *res = NULL;
    *row = *col = -1;
    return ;
   }
 stat = PQresultStatus(r);
 if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
   {
    sprintf(errstr,"PQexec:\n%s\n%s\nQuery was:\n%s",PQcmdStatus(r),PQresultErrorMessage(r), query);
    *res = NULL;
    *row = *col = -1;
    clear_result(&r);
    return;
   }
 *res = pgres2pc3(r, row, col);
 clear_result(&r);
}


void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
{
 int stat;
 va_list ap;
 char query[MAX_QUERY_LENGTH];
 va_start(ap, fmt);
 vsprintf(query,fmt,ap);
 va_end(ap);
 if (!c)
   {
    *r = NULL;
    strcpy(errstr, "Not connected to DB");
    return;
   }
 if (!query)
   {
    *r = NULL;
    strcpy(errstr, "NULL query passed to DB");
    return;
   }
 *r= PQexec(c, query);
 if (!*r)
   {
    sprintf(errstr,"PQresult is NULL %s\nQuery was:\n%s\n", PQerrorMessage(c), query);
    return ;
   }
 stat = PQresultStatus(*r);
 if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
   {
    sprintf(errstr,"PQexec:\n%s\n%s\nQuery was:\n%s",PQcmdStatus(*r),PQresultErrorMessage(*r), query);
    clear_result(r);
    *r = NULL;
    return;
   }
}


PGresult* execute_query(PGconn* c,  char* query)
{
 PGresult* r;
 int stat;
 debug("SQLquery: \"%s\"\n", query);
 if (!query)
   {
    ERR error("NULL query passed\n");
    return NULL;
   }
 if (!c) return NULL;
 r= PQexec(c, query);
 if (!r)
   {
    ERR error("PQresult is NULL %s\n", PQerrorMessage(c));
    return NULL;
   }
 stat = PQresultStatus(r);
 if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
   {
    ERR error("PQexec: %s %s",PQcmdStatus(r),PQresultErrorMessage(r));
    error("Failed query was: \"%s\"\n", query);
    return NULL;
   }
 return r;
}


char*** pgres2pc3(const PGresult* r, int* a, int* b)
{
 char*** result;
 int i,j;
 if (!r) return 0;
 i=j=0;
 debug("pgres2pc3: %p\n", (void*)r);
 *a = PQntuples(r);
 *b = PQnfields(r);
 /*printf("ntuples=%d,%d\n", *a,*b);*/
 result = (char***)malloc((*a)<<2);
 if (!result) { ERR error("malloc error\n"); return NULL; }
 for (i=0;i<*a;i++)
   {
    result[i] = (char**)malloc((*b)<<2);
    if (!result[i])
      {
       for (j=0;j<i;j++) free(result[i]);
       ERR error("malloc_i error!\n");
       return NULL;
      }
   }
 for (i=0;i<*a;i++)
 for (j=0;j<*b;j++)
   {
    result[i][j] = (char*)malloc(PQgetlength(r,i,j)+1);
    if (!result[i][j])
      {
       free_p3c(&result,*a,*b);
       ERR error("malloc_ij error\n");
       return NULL;
      }
    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
   }
#ifdef DEBUG
  debug("char*** result is (%d:%d):\n",i,j);
  for (i=0;i<*a;i++)
    {
     for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
     ln();
    }
#endif
 return result;
}


void free_p3c(char**** p, int a, int b)
{
 int i,j;
 debug("free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
 if (!(*p)) return ;
 for (i=0;i<a;i++) if (!(*p)[i]) return ;
 for (i=0;i<a;i++)
 for (j=0;j<b;j++) if ((*p)[i][j]) free((*p)[i][j]);
 for (i=0;i<a;i++) if ((*p)[i])    free((*p)[i]);
 free(*p);
}

